// -----------------------------------------------------
// module       : frm_io_fx.h
// purpose      : Implement headers for the "framework" set of basic I/O functions 
// revised      : 2014-02-26T15:51, added frm_ioDoesfileExist() frm_ioDoesDirectoryExist()
// revised      : 2014-02-16T17:41, added frm_ioWriteText()
// revised      : 2014-01-27T17:15:00-07:00, added filepath diagnostics to read,write binary functions
// history      : 2014-01-24T18:05:00-07:00, orig re-factor from MUM API
// history      : n/a
// author       : joe glassy, Lupine Logic Inc.
// dependencies : tbd
// history      : 2014-01-007T15:38:00, orig consolidation of MUM fx into one file
// notes        : Code originally in UM MODIS (MUM) API, adapted for L4_C framework context
// -----------------------------------------------------


#ifndef FRM_IO_DEF
#define FRM_IOL_DEF   1


// include all common, core symbols for frm_api
#include "frm_core.h"

// tag updated only for major milestones, not checkpoints
#define FRM_IO_VERS_TAG "v1.4-2014-07-31T16:01:00,jmg"
// name of unit test app
#define FRM_IO_APP_NAME "frm_io_fx.ex"


// -----------------------------------------------------
// FUNCTION PROTOTYPES
// -----------------------------------------------------
char *frm_ioQualifyPathWithRoot(const char *rootDirPath_p,char *origPath_p);

char *frm_ioGetIOModeLabel(const int ioModeIndex);

FILE *frm_ioOpenfile(const char *filePath_p,const char *callerLabel,const int tivIoMode);

int  frm_ioClosefile(FILE *filHndl,const char *filePath_p,const char *callerLabel);

void *frm_ioReadbinary(const char *inFilepath_p,const int tivDatatype,const size_t nElem,void *imgData_p);

int   frm_ioWritebinary(const char *outFilepath_p,const int tivDatatype,const size_t nElem,void *imgData_p);

// <JMG> added 2/16/2014, writes a single user custom formatted string to a (prior open) text output file
int frm_ioWriteText(FILE *outf,char *fmtSpecStr_p,...);

// passively scan a text file, return count of widest line and n. of lines
int  frm_ioGetTextfileWidth(const char *inputTextPath_p,int *lineCount);

size_t frm_ioGetFilesize(const char *inputPath_p,const char *callerLabel);

// read whole text file in one pass, parse, return as a IsStringTable. When options==1, ignores comment lines
char **frm_ioReadTextIntoTable(char *inputPath_p,const int options,int *maxWidthChars,int *nLines);
char *frm_ioReadTextIntoBuffer(char *inputPath_p);

// <JMG> 2015-09-24, added frm_ioWriteTexTable() to mirror ioReadTextIntoTable()
int  frm_ioWriteTextTable(char *outputPath_p,char **strTbl_p,int nLines,int opMode);

FILE *frm_Openfile(const char *filePath_p,const char *modeStr_p,char *caller_p);
int   frm_Closefile(FILE *filf,char *caller_p) ;

int frm_ioDoesDirectoryExist(const char *testDirectoryPath);

int frm_ioDoesFileExist(const char *testFilePath);

#endif
