// --------------------------------------------------------------------
// module   : pkg_mgr_prot.h
// purpose  : implement function prototypes for PkgMgr mini API
// revision : 2020-07-13t12:59 <JMG> at NTSG, changed more fx  to int64_t for hid_t compat.
// revision : 2020-07-08, changed int to int64_t for better hid_t compatibility in SetObjOwner,GetObjOwner
// revision : 2020-06-24, changed int to int64 for hid_t compatibility in SetObjOwner,GetObjOwner
// revision : 2014-09-12, added pkgMgr_GetObjH5NameByHdf5Id() and
//            pkgMgr_GetObjH5IdByName() and pkgMgr_DoesThemes_ChildNameExist()
// revision : 2014-08-13, added pkgMgr_GetNameByIndex()
// revision : 2014-07-23, added "get"|"set" for valid_min,valid_max
// revision : 2014-07-01, changed pkgMgr_InitObj() newDims type to size_t to match internal
// history  : 2014-06-26T16:06, added proto for singleton,
//            pkgMgr_CreateTheme() , and pkgMgr_ReleaseTheme()
// history  : 2014-06-11T16:10:04, added prototypes for Shortname,
//            Longname, GenericPath as PkgObj and PkgTheme support functions
//            and added  pkgMgr_SetThemeObjectCount() and pkgObj_ReleaseAllObjects()
// history  : 2014-02-26T17:04, added SetResolution,GetResolution
// history  : 2014-02-22T15:55, removed categoryString arg from
//            pkgMgr_CreateThemesPkgObjArray
// history  : 1.0-2014-01-15T14:24:00,jmg (original)
// version  : 1.0
// author   : joe glassy, Lupine Logic Inc.
// notes    : n/a
// --------------------------------------------------------------------

#ifndef PKG_MGRPROT_DEF 
#define PKG_MGRPROT_DEF 1

#include "pkg_mgr.h"

// --------------------------------------------------------------------
// module: pkg_mgr.c :: PkgMgr tier service functions
char   *pkgMgr_GetVersion(char *versBuffer_p);

int     pkgMgr_GlobalReport(PkgMgr *pkgMgr_p,FILE *outf,char *reportLabel_p);;

PkgMgr *pkgMgr_CreateMgr(const char *pkgmgrLabel,const int nThemes,const char *commonCategory_p);

void     pkgMgr_ReleaseMgr(PkgMgr *pkgMgr_p);

void pkgMgr_SetMgrName(PkgMgr *pkgMgr_p,const char *newPkgMgrLabel_p);
char *pkgMgr_GetMgrName(PkgMgr *pkgMgr_p);

// <JMG> 2014-08-14, added to retrieve MASTER list of ALL children across ALL themes
char **pkgMgr_GetMgr_AllNames(PkgMgr *pkgMgr_p,FILE *destf_p,int *masterCount) ;

// --------------------------------------------------------------------
// module: pkg_mgr_themes.c  namespace: all have a *Theme* in them
// PkgTheme tier service functions

// <JMG> 2014-09-12 added...
int pkgMgr_DoesThemes_ChildNameExist(PkgTheme *pkgTheme_p,const char *targetChildName_p);

// <JMG> 6/11/2014 added SetThemeCount
void  pkgMgr_SetThemeCount(PkgMgr *pkgMgr_p,const int newThemeCount);
int   pkgMgr_GetThemeCount(PkgMgr *pkgMgr_p);

void  pkgMgr_SetThemeNameByIndex(PkgMgr *pkgMgr_p,const int themeIndex,char *fmtStr_p,...);
char  *pkgMgr_GetThemeNameByIndex(PkgMgr *pkgMgr_p,const int themeIndex);
int   pkgMgr_GetThemeIndexByName(PkgMgr *pkgMgr_p,const char *themeName_p);

void *pkgMgr_GetMgr_ThemesChildDataByName(PkgMgr *pkgMgr_p,const char *themeLabel_p,
					const char *childLabel_p);
					
void *pkgMgr_GetMgr_ThemesChildDataByIndex(PkgMgr *pkgMgr_p,const int themeIdx,
					const int childIdx);

int pkgMgr_BindObjByName(PkgTheme *pkgTheme_p,const char *keyword,
			void *rawData_p);
			
int pkgMgr_InitTheme(PkgTheme *pkgTheme_p,const int themeId, const int ownerId,
			const char *themeLbl_p,const char *themeCategory_p);

// Within a given PkgTheme, creates (allocates) a fixed size collection
//   of nNewPkgObj PkgObjs. These are initially empty candidates.  
PkgTheme *pkgMgr_CreateThemes(PkgMgr *pkgMgr_p,const int nNewThemes,
				const int *nPkgObjList,const char *themeCategory);

// This is an alternative manual way to create the array of PkgObj that
// a given theme, thisTheme_p owns. Removed categoryString here since
// PkgObj don't inherit this quite yet.
PkgObj *pkgMgr_CreateThemesPkgObjArray(PkgTheme *thisTheme_p,
			const int nNewPkgObj) ;
					
// FUTURE? AddThemes is used to GROW the collection of themes a PkgMgr owns.
// a count larger than the current nMaxThemes causes the collection to grow.
// a count <= current nMaxThemes does nothing.
// void  pkgMgr_AddThemes(PkgMgr *pkgMgr_p,const int newTotalThemeCount);


// <JMG> 6/11/2014 added
void     pkgMgr_ReleaseAllObjects(PkgTheme *pkgTheme_p);
// <JMG> 6/26/2014 added, to support anonymous, un-bound PkgTheme's
void pkgMgr_ReleaseTheme(PkgTheme *pThme_p) ;
// legacy destructor
void     pkgMgr_ReleaseThemes(PkgMgr *pkgMgr_p);

PkgTheme *pkgMgr_GetThemeByIndex(PkgMgr *pkgMgr_p,const int themeIndex);
PkgTheme *pkgMgr_GetThemeByName(PkgMgr *pkgMgr_p,const char *themeName);

void     pkgMgr_SetThemeCategory(PkgTheme *pkgTheme_p,char *fmtStr_p,...);
		
char    *pkgMgr_GetThemeCategory(PkgTheme *pkgTheme_p);

void    pkgMgr_SetThemeName(PkgTheme *pkgTheme_p,char *fmtStr_p,...);
								
char    *pkgMgr_GetThemeName(PkgTheme *pkgTheme_p);

// <JMG> added 6/11/2014 as this was omitted earlier
void    pkgMgr_SetThemeObjectCount(PkgTheme *pkgTheme_p,const int newPkgObjCount);
int     pkgMgr_GetThemeObjectCount(PkgTheme *pkgTheme_p);


// <JMG> added 2014-02-17
void pkgMgr_SetThemes_ChildName(PkgTheme *pkgTheme_p,const int pkgObjIdx,char *newObjname_p);
char *pkgMgr_GetThemes_ChildName(PkgTheme *pkgTheme_p,const int pkgObjIdx);
// <JMG> added 2014-08-14, retrieves list of ALL names
char **pkgMgr_GetThemes_ChildListOfNames(PkgTheme *pkgTheme_p);

// get childs raw data, by index
void *pkgMgr_GetThemes_ChildData(PkgTheme *pkgTheme_p,const int pkgPbjIdx);

// get childs raw data, by name
void *pkgMgr_GetThemes_ChildDataByName(PkgTheme *pkgTheme_p,const char *pObjName);

// SET childs raw data, by index
void *pkgMgr_SetThemes_ChildData(PkgTheme *pkgTheme_p,const int pkgPbjIdx,void *rawData_p);

// SET childs raw data, by name. Just a convenience wrapper for 
//    int pkgMgr_BindObjByName() defined earlier
void pkgMgr_SetThemes_ChildDataByName(PkgTheme *pkgTheme_p,const char *objName,void *rawData_p);

// adeed 2014/02/25..jmg
void pkgMgr_SetThemes_ChildMetadata(PkgTheme *pkgTheme_p,const int pkgObjIdx,char *newMetadata_p);
char *pkgMgr_GetThemes_ChildMetadata(PkgTheme *pkgTheme_p,const int pkgObjIdx);
char *pkgMgr_GetThemes_ChildMetadataByName(PkgTheme *pkgTheme_p,const char *pObjName);
void pkgMgr_SetThemes_ChildMetadataByName(PkgTheme *pkgTheme_p,const char *pObjName,char *newMetadata_p);
// added 3/7/2014
void pkgMgr_SetThemes_ChildIdByName(PkgTheme *pkgTheme_p,const char *pObjName,const int64_t newId);
int64_t  pkgMgr_GetThemes_ChildIdByName(PkgTheme *pkgTheme_p,const char *pObjName);
// added 6/11/2014
void pkgMgr_SetThemes_ChildGenericPathByName(PkgTheme *pkgTheme_p,const char *pObjName,char *newGenPath_p);
char *pkgMgr_GetThemes_ChildGenericPathByName(PkgTheme *pkgTheme_p,const char *pObjName);


// --------------------------------------------------------------------
// module: pkg_mgr_objects.c  namespace: all have a *Obj* in their name

void    pkgMgr_ObjBind(PkgObj *pkgObj_p,void *referent_p);

void    *pkgMgr_GetObjData(PkgObj *pkgObj_p);
void    pkgMgr_ClearObject(PkgObj *pkgObj_p) ;

// <JMG> 2014-08-14T17:16, added basic hid_t entities for minimal HDF5 support
void  pkgMgr_SetObjH5DatasetId(PkgObj *pkgObj_p,hid_t newId);
hid_t pkgMgr_GetObjH5DatasetId(PkgObj *pkgObj_p);

void  pkgMgr_SetObjH5Datatype(PkgObj *pkgObj_p,hid_t newHdf5Datatype);
hid_t pkgMgr_GetObjH5Datatype(PkgObj *pkgObj_p);

void  pkgMgr_SetObjH5Dcpl(PkgObj *pkgObj_p,hid_t newDcpl);
hid_t pkgMgr_GetObjH5Dcpl(PkgObj *pkgObj_p);

void  pkgMgr_SetObjH5Dspace(PkgObj *pkgObj_p,hid_t newDspace);
hid_t pkgMgr_GetObjH5Dspace(PkgObj *pkgObj_p);

// <JMG> 2014-09-12T11:05:00, added, to support HDF5 lookups
char *pkgMgr_GetObjH5NameByHdf5Id(PkgTheme *pkgTheme_p,const hid_t h5Id,char *resultName_p);
hid_t pkgMgr_GetObjH5IdByName(PkgTheme *pkgTheme_p,const hid_t h5Id,char *targetName_p);

// <JMG> 2014-06-26T16:04, changed to supply objOwnerId not PkgTheme
PkgObj *pkgMgr_CreateObj(const int64_t objOwnerId,const char *keywordName_p,
		const int64_t objID,const int tivDatatype);
		
void    pkgMgr_ReleaseObj(PkgObj *pkgObj_p);

// <JMG> 2020-06-24T15:35 changed int to int64_t for hid_t compatibility
void     pkgMgr_SetObjId(PkgObj *pkgObj_p,const int64_t newObjId);
int64_t  pkgMgr_GetObjId(PkgObj *pkgObj_p);

// <JMG> 2020-06-24, trial adjust datatypes to int64_t for better hid_t compatibility
void    pkgMgr_SetObjOwner(PkgObj *pkgObj_p,const int64_t newOwnerId);
// <JMG> 2020-06-24, trial adjust datatypes to int64 for hid_t compatibility
int64_t   pkgMgr_GetObjOwner(PkgObj *pkgObj_p);

void    pkgMgr_SetObjDatatype(PkgObj *pkgObj_p,const int tivDatatype);
int     pkgMgr_GetObjDatatype(PkgObj *pkgObj_p);

void   pkgMgr_SetObjNelem(PkgObj *pkgObj_p,const size_t nElem);
size_t pkgMgr_GetObjNelem(PkgObj *pkgObj_p);

void    pkgMgr_SetObjRank(PkgObj *pkgObj_p,const int newRank);
int     pkgMgr_GetObjRank(PkgObj *pkgObj_p);

void   pkgMgr_SetObjCategoryCode(PkgObj *pkgObj_p,const int newCategoryCode);
int    pkgMgr_GetObjCategoryCode(PkgObj *pkgObj_p);

void    pkgMgr_SetObjDims(PkgObj *pkgObj_p,const size_t *newDims);
size_t  *pkgMgr_GetObjDims(PkgObj *pkgObj_p);

int     pkgMgr_GetIdByName(PkgTheme *pkgTheme_p,const char *targetObjName);

// <JMG> 2020-07-13, change to int64_t for new int64_t hid_t compat
char    *pkgMgr_GetNameById(PkgTheme *pkgTheme_p,const int64_t uniqueId,
						char *resultObjName);
						
// <JMG> 2014-08-013, added: pkgMgr_GetNameByIndex()
char *pkgMgr_GetNameByIndex(PkgTheme *pkgTheme_p,const int internalIndex,char *resultObjName);

// VERIFY: does not now exist.
// PkgObj  *pkgMgr_GetObj(PkgTheme *pkgTheme_p,void *dataPtr_p);

PkgObj  *pkgMgr_GetObjByName(PkgTheme *pkgTheme_p,const char *objName_p);

PkgObj  *pkgMgr_GetObjByIndex(PkgTheme *pkgTheme_p,const int ithIndex);

int     pkgMgr_GetObjIndexByName(PkgTheme *pkgTheme_p,const char *targetObjName);

void    pkgMgr_SetObjName(PkgObj *pkgObj_p,char *fmtStr_p,...);
char    *pkgMgr_GetObjName(PkgObj *pkgObj_p);

void    pkgMgr_SetObjFillvalue(PkgObj *pkgObj_p,const double fillVal);
double  pkgMgr_GetObjFillvalue(PkgObj *pkgObj_p);

char *pkgMgr_GetObjUnits(PkgObj *pkgObj_p);
void  pkgMgr_SetObjUnits(PkgObj *pkgObj_p,const char *unitStr_p);

// <JMG> 2014-07-23, added valid_min,valid_max
void   pkgMgr_SetObjValidMin(PkgObj *pkgObj_p,const double validMin);
double pkgMgr_GetObjValidMin(PkgObj *pkgObj_p) ;
void   pkgMgr_SetObjValidMax(PkgObj *pkgObj_p,const double validMax);
double pkgMgr_GetObjValidMax(PkgObj *pkgObj_p) ;
//
void   pkgMgr_SetObjGain(PkgObj *pkgObj_p,const double gainVal);
double pkgMgr_GetObjGain(PkgObj *pkgObj_p);
void   pkgMgr_SetObjOffset(PkgObj *pkgObj_p,const double offsetVal);
double pkgMgr_GetObjOffset(PkgObj *pkgObj_p);


// <JMG> 2014-08-06, added {Set|Get}ValidMin,ValidMax,ValidRange
void pkgMgr_SetObjValidRange(PkgObj *pkgObj_p,const double valMin,const double valMax);
double pkgMgr_GetObjValidRange(PkgObj *pkgObj_p,double *valMin,double *valMax);

// <JMG> 2014-07-01, changed newDims type to size_t to match internal
int pkgMgr_InitObj(PkgObj *pkgObj_p,const char *objKeyword_p,
	const int64_t objID,const int64_t objOwnerId,
	const int tivDatatype,const int newCategory, const int oRank, const size_t *newDims);

// added 2/25/2014..
void pkgMgr_SetObjMetadata(PkgObj*pkgObj_p,char *pkObjectMetadata_p);
char *pkgMgr_GetObjMetadata(PkgObj*pkgObj_p);
// added 2/26/2014
void pkgMgr_SetObjResolution(PkgObj *pkgObj_p,const int resolutionCode);
int  pkgMgr_GetObjResolution(PkgObj *pkgObj_p);
// added 3/5/2014
void   pkgMgr_SetObjSelectFilter(PkgObj *pkgObj_p,const uint8 selectionFilterBoolean);
uint8  pkgMgr_GetObjSelectFilter(PkgObj *pkgObj_p);

// added 06/11/2014
void pkgMgr_SetObjShortname(PkgObj*pkgObj_p,char *newShortname_p);
char *pkgMgr_GetObjShortname(PkgObj*pkgObj_p);
void pkgMgr_SetObjLongname(PkgObj*pkgObj_p,char *newLongname_p);
char *pkgMgr_GetObjLongname(PkgObj*pkgObj_p);
void pkgMgr_SetObjGenericPath(PkgObj*pkgObj_p,char *newGenericPath_p);
char *pkgMgr_GetObjGenericPath(PkgObj*pkgObj_p);

// added 08/13/2014 for better list-wise processing
int pkgMgr_GetThemes_ChildIndexByName(PkgTheme *pkgTheme_p,const char *pObjName) ;

// added 3/7/2014
void pkgMgr_SetThemes_ChildIdByName(PkgTheme *pkgTheme_p,const char *pObjName,const int64_t newId);
int64_t pkgMgr_GetThemes_ChildIdByName(PkgTheme *pkgTheme_p,const char *pObjName);

void pkgMgr_SetMgr_ThemesChildIdByName(PkgMgr *pkgMgr_p,const char *themeLabel_p,
					const char *childLabel_p,const int64_t newId);
// <JMG> changed 2020-07-13T12:58 for hid_t compatibility
int64_t pkgMgr_GetMgr_ThemesChildIdByName(PkgMgr *pkgMgr_p,const char *themeLabel_p,
					const char *childLabel_p);
					
void pkgMgr_SetMgr_ThemesChildOwnerByName(PkgMgr *pkgMgr_p,const char *themeLabel_p,
					const char *childLabel_p,const int newOwnerId);
					
int pkgMgr_GetMgr_ThemesChildOwnerByName(PkgMgr *pkgMgr_p,const char *themeLabel_p,
					const char *childLabel_p);

void pkgMgr_SetThemes_ChildDatatypeByName(PkgTheme *pkgTheme_p,const char *pObjName,const int newDatatype);
int pkgMgr_GetThemes_ChildDatatypeByName(PkgTheme *pkgTheme_p,const char *pObjName);
#endif
