// ------------------------------------------------------------
// module       : frm_core.c
// purpose      : Implement FRM (Framework) API "core" functions. Key goal of
//                these are purely administrative, e.g. return version number or
//                related functionality not covered under the API proper
// revision     : 2016-02-18T08:58, freshened docs for match to L4CMDL v7.1.0 delivery
// revision     : 2014-02-25T14:27:00,added pkgMgr_SetObjMetadata(),pkg_SetThemeMetadata(),
//                pkg_SetObjCategoryCode(), etc.
// revision     : 2014-02-21T12:03:00, added pkgMgr_GlobalReport()
// history      : 2014-02-09T20:16:00-07:00, added frm_GetApiVersion(), frmStrblkData labels

// history      : 2014-01-27T17:14:00-07:00, added filepath diagnostics to read,write binary functions
// history      : 2014-01-27T17:04:00,wheat,jmg
// version      : see macro FRM_IO_VERS_TAG in header file frm_io_fx.h
// author       : joe glassy, Lupine Logic Inc.
// dependencies : verify, HDF5, HDF5 etc
// notes        : n/a


#include "frm_api.h"

// Currently: 23 datatype identifiers are supported. Although shown in
//  a particular case, they are case insensitive when used, e.g. all
//  auto-converted to all lowercase.
static const char *frmStrblkDatatypeLabels[] = {
 "NoType"    ,
 "Int8"      ,
 "Uint8"     ,
 "Int16"     ,
 "Uint16"    ,
 "Int32"     ,
 "Uint32"    ,
 "Int64"     ,
 "Uint64"    ,  
 "Float32"   ,
 "Float64"   ,
 "SizeT"     ,
 "TimeT"     ,
 "Boolean"   ,
 "Datetime"  ,
 "Directoryspec",
 "Filespec"   , 
 "Integer"    ,
 "Real"       , 
 "String"     ,
 "StringTable",
 "StringList", 
 "CustomUserType"
} ;


// ------------------------------------------------------------
// function     : frm_GetApiVersion()
// inputs       : char *versionStr_p --ptr to callers string to store VERS_TAG in
// purpose      : Retrieves Framework API's complete version tag 
// returns      : char * --ptr to API version identifier string.
// revision     : 2014-02-01T10:11:00-07:00,orig
// history      : n/a
// author       : joe glassy, Lupine Logic Inc.
// notes        : n/a
char *frm_GetApiVersion(char *versionIdentifier_p)
{

 // capture current version tag stored in frm_core.h 
 strcpy(versionIdentifier_p,(char *)FRM_CORE_VERS_TAG);
 
 // Return the version tag that includes the version (major.minor) and 
 //   release date this milestoned version is associated with.
 
 return versionIdentifier_p ;
 
} // end::frm_GetApiVersion()


#ifdef FRM_API_CORE_UNITTEST

int main(int argc,char *argv[])
{
 char versionStr[32] = {0} ;
 
	// Retrieve current version tag and report to caller
	frm_GetApiVersion(versionStr);
	
	fprintf(stdout,"\n");
	fprintf(stdout,"\n");
	fprintf(stdout,"l4c Framework API current version is [%s]\n",versionStr);
   
   return 0;	
	
} // end::main()
#endif

