#
#  module       : l4c_MET.mak -- makefile for SMAP l4c_MET.ex
#  purpose      : Build executable for SMAP L4_C preprocessor l4c_MET.ex
#  revision     : 2014-10-08T16:33:00, for D5 v5.0.1-R11E for ORT, ORR, etc.
#  history      : 2014-07-23T10:10:00-07:00, for D5+ (v5.0.1)
#  history      : 2013-09-23T14:09:00:00-07:00,D4
#  version      : v4.6.3+
#  author       : Joe Glassy
#  dependencies : HDFv5 libraries. Note that libszip is typicaly referenced but is NOT currently required
#               : headers: l4c_MET.h, l4c_MET_prot.h
#  notes        : A different library mix is required to build at NCCS.

#  build syntax (build platform Barley): gcc $(CFLAGS) -o l4c_MET.ex l4c_MET.c $(INC) $(LIBSHDF) $(LIB)
#
# gcc -g -Wall -Wextra -std=c99 -D_GNU_SOURCE  -o l4c_daily.ex L4C_bplut.o array_io.o timer.o tcf_io_params.o
#    tcf_mod_mult.o tcf_mod_func.o tcf_mod_drivers_grid.o tcf_run_mode_mgr.o l4c_daily_io.o l4c_log_svc.o 
#    l4c_rip_str.o l4c_mem.o l4c_init_mgr.o l4c_daily.o -I../inc -I/usr/include/hdf -I/usr/include 
#     -L../lib -L/usr/lib64/hdf -L/usr/lib64  -lhdf5 -lhdf5_hl -lmfhdf -ldf -ljpeg -lz -lsz -lm -lutil 

CC = gcc

#  -DHAVE_UUID -DHAVE_HDF4 -DHAVE_HDF5
CFLAGS =  -g -Wall -Wextra -std=c99 -D_GNU_SOURCE -DHAVE_UUID -DHAVE_HDF4 -DHAVE_HDF5

FRM_API_SRC = $(FRM_API_ROOT)/src
# in case these aren't explicitly defined we define these here:
# FRM_API_INC = $(FRM_API_ROOT)/inc
# FRM_API_LIB = $(FRM_API_ROOT)/lib
 
# syntax from l4c_DAILY.mak, that should also work at NCCS
LDFLAGS = -lfrm -ldf -lmfhdf -lhdf5 -lhdf5_hl -ljpeg -lz -lsz -luuid -lm

# Link in both HDFv4 and HDFv5 libraries. Now added -luuid for uuid generation
# requires access to ./lib64/libuuid.a|.so and to <uuid/uuid.h> also.
# LIBFLAGS = -lfrm -lhdf5 -lhdf5_hl -lmfhdf -ldf -ljpeg -lz -lsz -lefence -luuid -lm

HDF4INC=/usr/include/hdf
HDF4LIB=/usr/lib64/hdf

HDF5INC=/usr/include
HDF5LIB=/usr/lib64

# LIBDIR = ../lib
LIBDIR = .
# INCDIR = ../inc
INCDIR = .

INCS = -I$(FRM_API_INC) -I$(INCDIR) -I$(HDF4INC) -I$(HDF5INC) 

LIBS = -L$(LIBDIR) -L$(HDF4LIB) -L$(HDF5LIB) -L$(FRM_API_LIB)

TARGET = l4c_MET.ex

SRCS =  l4c_MET_rip.c \
	l4c_MET_ses.c \
	l4c_MET_fx.c  \
	l4c_MET_io.c \
	l4c_MET_meta.c \
	l4c_MET_grid_proj.c \
	l4c_MET_PkgMgrInit.c \
	l4c_MET_freezethaw.c \
	l4c_MET_drilldowns.c \
	l4c_MET_drivers.c \
	l4c_MET.c


# Instantiate one object file per listed .c source file
OBJS = $(SRCS:.c=.o)

all: $(TARGET) 

$(TARGET): $(OBJS)
	$(CC) $(CFLAGS) -o $(TARGET) $(OBJS) $(INCS) $(LIBS) $(LDFLAGS) 

clean:
	rm -f *.o *~ core *.ex  $(INCDIR)*~ 

# TBD: figure out how to vary the .rip file based on platform name!
test:
	./$(TARGET) l4c_METs.rip
	h5dump -H SMAP_L4_C_MET.h5 | grep DATASET


#Update the .o files whenever a newer .c file is encountered
.c.o:
	$(CC) $(CFLAGS) $(INCS) -c $< -o $@

