#
#  module       : l4c_MET_UTIL.mak -- makefile for a stand-alone MET checking utility
#  purpose      : Build a stand-alone MET assessment utility.
#  revision     : 2015-08-10T16:14:00
#  history      : n/a
#  version      : v1.0
#  history      : n/a
#  author       : Joe Glassy
#  dependencies : HDFv5 libraries. Note that libszip is typicaly referenced but is NOT currently required
#               : headers: l4c_MET.h, l4c_MET_prot.h
#  notes        : A different library mix is required to build at NCCS.

#  build syntax (build platform Barley): gcc $(CFLAGS) -o l4c_MET.ex l4c_MET.c $(INC) $(LIBSHDF) $(LIB)

CC = gcc

#  -DHAVE_UUID -DHAVE_HDF4 -DHAVE_HDF5
CFLAGS =  -g -Wall -Wextra -std=c99 -D_GNU_SOURCE -DHAVE_UUID -DHAVE_HDF4 -DHAVE_HDF5

FRM_API_SRC = $(FRM_API_ROOT)/src
# TBD: check, in case these aren't explicitly defined we define these here:
# FRM_API_INC = $(FRM_API_ROOT)/inc
# FRM_API_LIB = $(FRM_API_ROOT)/lib
 
# syntax from l4c_DAILY.mak, that should also work at NCCS
LDFLAGS = -lfrm -ldf -lmfhdf -lhdf5 -lhdf5_hl -ljpeg -lz -lsz -luuid -lm

# Link in both HDFv4 and HDFv5 libraries. Now added -luuid for uuid generation
# requires access to ./lib64/libuuid.a|.so and to <uuid/uuid.h> also.
# LIBFLAGS = -lfrm -lhdf5 -lhdf5_hl -lmfhdf -ldf -ljpeg -lz -lsz -lefence -luuid -lm

HDF4INC=/usr/include/hdf
HDF4LIB=/usr/lib64/hdf

HDF5INC=/usr/include
HDF5LIB=/usr/lib64

# LIBDIR = ../lib
LIBDIR = .
# INCDIR = ../inc
INCDIR = .

INCS = -I$(FRM_API_INC) -I$(INCDIR) -I$(HDF4INC) -I$(HDF5INC) 

LIBS = -L$(LIBDIR) -L$(HDF4LIB) -L$(HDF5LIB) -L$(FRM_API_LIB)

TARGET = l4c_MET_UTIL.ex

SRCS =  l4c_MET_util.c 

# Instantiate one object file per listed .c source file
OBJS = $(SRCS:.c=.o)

all: $(TARGET) 

$(TARGET): $(OBJS)
	$(CC) $(CFLAGS) -o $(TARGET) $(OBJS) $(INCS) $(LIBS) $(LDFLAGS) 

clean:
	rm -f *.o *~ core *.ex  $(INCDIR)*~ 

# TBD: figure out how to vary the .rip file based on platform name!
test:
	./$(TARGET) 


#Update the .o files whenever a newer .c file is encountered
.c.o:
	$(CC) $(CFLAGS) $(INCS) -c $< -o $@

