// ---------------------------------------------------------------------
//   Module     : l4c_MET_prot.h
//   Purpose    : function prototypes for l4c_MET.c developmental prototype
//   Author     : joe glassy
//   Version    : 7.0 (see AppVers)
//   revision   : 2015-08-25T16:58:00, update, added calcNativeRange() and
//                changed name of write_FtStateUM_3km()
//   revision   : 2015-07-31T12:36:00, added support for GEOS FP TS 3KM F/T freeze-thaw
//   history    : 2014-10-21T20:10, added l4c_CalcVpd() and
//                added l4c_writeNvpAsAttribute()
//   history   : 2014-08-26T10:33, made L4C_MET_VERS_TAG consistent across
//                all executable components and added l4c_GetExecutableVersion()
//   history   : 2014-07-30T15:32:00, split out metadata into l4c_MET_meta.c
//   history   : 2014-07-25T19:23:00
//   Invocation : ./l4c_MET.ex <l4c_MET.rip> {<model_day_of_year>}
//   Version    : see AppVers macro below
// ---------------------------------------------------------------------


#ifndef L4C_MET_ProtDefs
#define L4C_MET_ProtDefs  1


#include "l4c_MET.h"

// IMPORT NSB prototypes, a component of the UM Framework 
#include "nsb_api_prot.h"

// IMPORT PkgMgr prototypes, a component of the UM Framework 
#include "pkg_mgr_prot.h"


// FUNCTION PROTOTYPES

// ---------------------------------------------------------------------
// l4c_MET.c (main)
int main(int argc,char *argv[]);


// ---------------------------------------------------------------------
// MODULE: l4c_MET_ses.c (session logic, added 2014-05-15,jmg)

char *l4c_GetExecutableVersionId(void) ; // <JMG> added 2014-08-26T10:33:00

SesCtl *l4c_GetSesCtl(void);
NsbObj *l4c_GetNsbOverseer(void);
PkgMgr *l4c_GetPkgMgrOverseer(void);
ImgPkg *l4c_GetImagePackage(void);

int32  l4c_InitSession(int argc,char *argv[],SesCtl *sesCtl_p);
int32  l4c_CloseSession(SesCtl *sesCtl_p) ;
double l4c_EventCompletionMsg(const char *thisFunction_p);
double l4c_EventHighWaterMarkMsg(const char *thisFunction_p);


// ---------------------------------------------------------------------
// l4c_MET_rip.c support functions
char *l4c_SesQualifyPathWithRoot(const char *rootDirPath_p,char *origPath_p);
void  l4c_ApplyRootDirToPaths(SesCtl *sesCtl_p);
void  l4c_SesVerifyNvps(NsbObj *nsb_p);

// CORE Nvps are retrieved using this standardized "Core" function
void  l4c_SesCollectCoreNvps(SesCtl *sesCtl_p);
// APP-SPECIFIC Nvps are retrieved using l4c_SesCollectNvps()
void  l4c_SesCollectNvps(SesCtl *sesCtl_p);

// ---------------------------------------------------------------------
// l4c_MET_sci.c support functions <JMG> 2014-10-21T20:10 added
float32 l4c_MET_CalcVPD(const float32 tavg,const float32 qv2m,const float32 ps);
int     l4c_MET_InitForVPD(void);

// ---------------------------------------------------------------------
// Module: l4c_MET_PkgMgrInit.c support functions
int l4c_PkgInit_SetFileProperties(void);
int l4c_PkgMgr_Init(const int nThemes);
void l4c_PkgInit_SMAP_L4_C_Ancillary(PkgTheme *pkgTheme_p,char *label); 
void l4c_PkgInit_SMAP_L4_C_SOC(PkgTheme *pkgTheme_p,char *label);
void l4c_PkgInit_SMAP_L4_SM(PkgTheme *pkgTheme_p,char *label);
void l4c_PkgInit_SMAP_L3_SM_A(PkgTheme *pkgTheme_p,char *label);
void l4c_PkgInit_GEOS5_FP(PkgTheme * pkgTheme_p,char *label);
void l4c_PkgInit_L4_C_MET(PkgTheme *pkgTheme_p,char *label);


// ---------------------------------------------------------------------
// Module: l4c_MET_meta.c
void l4c_WriteFile_metadata(SesCtl *sesCtl_p,hid_t file_id) ;
void l4c_WriteDataset_metadata(SesCtl *sesCtl_p,char *datasetName_p,hid_t dataset_id) ;
// <JMG> 2014-10-21, added (borrowed from l4c_DAILY.ex
hid_t l4c_writeNvpAsAttribute(hid_t file_id,const char *keyword,const char *attrForceName);

// ---------------------------------------------------------------------
// l4c_MET_io.c support functions
int   write_FtStateUM_3km(SesCtl *sesCtl_p,const char *datasetName, 
                         uint8 *ftStateUm_M03_Img_p);

int32 readPftMask3km(ImgPkg *imagePkg_p);
int32 readPftMask9km(ImgPkg *imagePkg_p);
int32 readAndWriteLatLon_9kmCoordVars(ImgPkg *imagePkg_p);
void l4c_PrepOutput_HDF5_Datasets(SesCtl *sesCtl_p) ;

void *fp_get_dataset(hid_t file_id,char *datasetName,int *dRank,hsize_t *dims);

// TBD, later, replace these local "HDF5 services" with
//  FRM_API versions of these as API instances are made available
hid_t createMetHdf5(SesCtl *sesCtl_p);
hid_t createMetHdf5Dataset(SesCtl *sesCtl_p,const int32 dsGridRes,
          const char *datasetName,const hid_t dsType_id,void *dsData_p);
hid_t createSOCHdf5Dataset(SesCtl *sesCtl_p,const int32 dsGridRes,
          const char *datasetName,const hid_t dsType_id,void *dsData_p);

// ---------------------------------------------------------------------
// l4c_MET_fx.c support functions
int    l4c_SummarizeFreezeThaw(SesCtl *sesCtl_p, uint8 *ftData_p,const size_t nElem3km);
char * l4c_GenerateUUID(char *resultUUID_String,const int displayMode) ;
int32 *genTerrestrialAddresses(uint8 *pftImg_p,const int32 nElem9km);
char *l4c_to_upper(char *targString);
// current single generic, may have to specialize by input file's variant
int32 granuleHHMM_to_HrIndex(char *granulePath,char *hhmmString,char *yyyyStr);

int32 showHourly_stats(const char *datasetName,const int32 nGranules,
                float32 *hrMinAry,float32 *hrMaxAry,float32 *hrMeanAry);

size_t calcNativeRange(SesCtl *sesCtl_p,char *datasetName,
        const size_t nPixels, const float32 missingValue,
        const float32 lowestValid, float32 *fltRaster_p ) ;

// <JMG> 2015-09-04 new
float32 *rescaleFloatImage(float32 *srcFltImg_p,
         const size_t srcRowCount,const size_t srcColCount,
         const size_t destRowCount,const size_t destColCount);
uint8 *rescaleByteImage(uint8 *srcByteImg_p,
         const size_t srcRowCount,const size_t srcColCount,
         const size_t destRowCount,const size_t destColCount);

// <JMG> 2015-09-04T13:27 added new
size_t calculateGranuleActualRange(SesCtl *sesCtl_p,char *datasetName,
                const float32 missingVal,float32 *varImg_p,size_t nElem);

// ---------------------------------------------------------------------
// l4c_MET_grid_proj.c support functions
int32 l4c_easegrid2_wgs84_fwd(const int32 mapProjectionCode,
   const float64 lat,const float64 lon, float64 *ithRow, float64 *ithColumn);
int32 l4c_easegrid2_wgs84_inv(const int32 mapProjectionCode,
   float64 ithRow, float64 ithColumn,float64 *lat,float64 *lon);
void  geog_forwardtransform(const float64 lat,const float64 lon, 
   int32 *ithRow, int32 *ithCol);
void  geog_inverse(const int32 ithRow,const int32 ithCol,
    float64 *lat,float64 *lon);


// 2015-07-31, added for GEOS5 FP TS 3KM F/T substitution support

// NO LONGER NEEDED, soon will deprecate
// int32 regrid_FP_to_EG_3km(const int32 nColsGeog, uint8 *pftImg3km_p,uint8 *destEgImg_p, uint8 *srcGeogImg_p);

int32 regrid_FP_to_EG_9km(const int32 nColsGeog,uint8 *pftImg9km_p, float32 *destEgImg_p,float32 *srcGeogImg_p);


// ---------------------------------------------------------------------
// l4c_MET_drivers.c 
int32 driverLogic_GEOS5_FP_Meteorology(SesCtl *sesCtl_p,char *dayOfYearStr);
// L4_SM specific...
int32  driverLogic_L4_SM(SesCtl *sesCtl_p  ,char *dayOfYearStr);
// L3_SM_A specific service functions
int32 driverLogic_L3_SM_A(SesCtl *sesCtl_p,char *dayOfYearStr);

// ---------------------------------------------------------------------
// <JMG> 2015-09-06, changed to return int only as FT_STATE_UM_M03 array now held in
// imagePkg_p package
int   l3_sm_a_ProcessFreezeThaw(SesCtl *sesCtl_p,uint16 *surfflag_p,uint8  *freezeThawQC_p);

// ---------------------------------------------------------------------
// l4c_MET_drilldowns.c support functions
int32 geos5_fp_MeteorologyDrilldown(SesCtl *sesCtl_p,char *dayOfYearStr,char *datasetName);
// L4_SM specific
int32 l4_sm_verticalDrilldown(SesCtl *sesCtl_p,char *dayOfYearStr,char *datasetName) ;
// GEOS5 FP "TS" specific handler for fallback
hid_t handler_geos5_ts_Fallback(SesCtl *sesCtl_p,float32 *tsEg9kmImg_p,char *dayOfYearStr);
// GEOS5 FP "T2M" MIN and MAX handlers
hid_t handler_geos5_T2M_Min(SesCtl *sesCtl_p,char *datasetName,float32 *eg9kmImg_p,float32 *vMin);
hid_t handler_geos5_T2M_Max(SesCtl *sesCtl_p,char *datasetName,float32 *eg9kmImg_p,float32 *vMax);
// GEOS5 FP "QV2M" handler
hid_t handler_geos5_QV2M(SesCtl *sesCtl_p,char *datasetName,float32 *var9kmImg_p,char *dayOfYearStr);
// GEOS-5 FP specific service functions
// new refactored rowReversal()
void *rowReversal(float32 *src_p,const int32 nRows,const int32 nCols);

int32 fp_granuleHHMM_to_HrIndex(char *granulePath,char *hhmmString,char *yyyyStr);

// for GEOS-5 FP, returns hourly mean across 2D spatial domains
float32 fp_summarizeVariable(const float32 allowableMin,
  const float32 allowableMax, char *datasetName,hsize_t *dims,
  float32 *dsNativeInputArray_p, float32 *hourlyMin,float32 *hourlyMax);        

// L4_SM specific service functions
int32 l4_sm_granuleHHMM_to_HrIndex(char *granulePath,char *hhmmString,char *yyyyStr);
void *l4_sm_get_dataset(hid_t file_id,char *datasetName,int *dRank,hsize_t *dims,int32 *nPix) ;
// for L4_SM, returns hourly mean across 2D spatial domains
float32 l4_sm_summarizeVariable(FILE *logf,char *datasetName,hsize_t *dims,float32 *dsData_p,float32 *hourlyMin,float32 *hourlyMax) ;

// L3_SM_A specific service functions
// <JMG> 7/30/2014, changed return type to explicit uint16*
uint16 *l3_sm_a_get_surface_flag(SesCtl *sesCtl_p,char *datasetName,int *dRank,hsize_t *dims);
int32 l3_sm_a_granuleHHMM_to_HrIndex(char *granulePath,char *hhmmString,char *yyyyStr);


#endif
