// ---------------------------------------------------------------------
// l4c_MET_prot.h
//   Purpose    : function prototypes for l4c_MET.c developmental prototype
//   Author     : joe glassy
//   Version    : 5.0.1 (see AppVers)
//   Invocation : ./l4c_MET.ex <l4c_MET.rip> {<model_day_of_year>}
//   Version    : see AppVers macro below
// ---------------------------------------------------------------------


#ifndef L4C_MET_ProtDefs
#define L4C_MET_ProtDefs  1


#include "l4c_MET.h"


// FUNCTION PROTOTYPES

// ---------------------------------------------------------------------
// l4c_MET.c (main)
int main(int argc,char *argv[]);

// ---------------------------------------------------------------------
// MODULE: l4c_MET_ses.c (session logic, added 2014-05-15,jmg)
SesCtl *l4c_GetSesCtl(void);
NsbObj *l4c_GetNsbOverseer(void);
PkgMgr *l4c_GetPkgMgrOverseer(void);

void  l4c_SesVerifyNvps(NsbObj *nsb_p);
void  l4c_SesCollectNvps(SesCtl *sesCtl_p, NsbObj *nsb_p);

int32 l4c_InitSession(int argc,char *argv[],SesCtl *sesCtl_p);
int32 l4c_CloseSession(SesCtl *sesCtl_p) ;


// ---------------------------------------------------------------------
// l4c_MET_fx.c support functions
char * l4c_GenerateUUID(char *resultUUID_String,const int displayMode) ;
uint8 *readPftMask9km(char *pftMaskPath,const int32 nElem_9km);
int32 *genTerrestrialAddresses(uint8 *pftImg_p,const int32 nElem9km);
char *l4c_to_upper(char *targString);
// current single generic, may have to specialize by input file's variant
int32 granuleHHMM_to_HrIndex(char *granulePath,char *hhmmString,char *yyyyStr);
// TBD : stubs put in for now, need to implement each of these as needed

// Generic support functions

// bit manip
uint8 l4c_testByteorder(void);
unsigned int l4c_GetBits(unsigned int xTarget, const unsigned int pStart,const unsigned int nBits);
uint32 l4c_SetBits(int16 startPos, int16 endPos, void *data, size_t dataSize);
// alternative style of GetBits
uint32 l4c_RetrieveBits(const int16 startPos,const int16 endPos, void *data,size_t dataSize);

int32 logMsg(FILE *f,char *msg,...);
// parse on std RIP keyword=value NVP syntax
int32 parseRipLines(SesCtl *sesCtl_p);		
int32 showHourly_stats(FILE *logf,const char *datasetName,const int32 nGranules,float32 *hrMinAry,float32 *hrMaxAry,float32 *hrMeanAry);

// HDF5 services
hid_t createMetHdf5(SesCtl *sesCtl_p);
hid_t createMetHdf5Dataset(SesCtl *sesCtl_p,const int32 dsGridRes,const char *datasetName,const int32 dsType_id,void *dsData_p);
hid_t createSOCHdf5Dataset(SesCtl *sesCtl_p,const int32 dsGridRes,const char *datasetName,const int32 dsType_id,void *dsData_p);
hid_t createHdf5Attribute(const hid_t owner_id,const char *attrLabel,const char *attrValue);

// TBD: later, replace all variant-specific HDF5 file open calls with just this generic one:
hid_t open_input_hdf5_granule(char *hdf5Path,char *variantLabel);

// GEOS-5 FP specific service functions
// new refactored rowReversal()
void *rowReversal(float32 *src_p,const int32 nRows,const int32 nCols);
int32 fp_granuleHHMM_to_HrIndex(char *granulePath,char *hhmmString,char *yyyyStr);
void *fp_get_dataset(hid_t file_id,char *datasetName,int *dRank,hsize_t *dims);
hid_t fp_geos5_openfile(char *hdf5Path);
// for GEOS-5 FP, returns hourly mean across 2D spatial domains
float32 fp_summarizeVariable(char *datasetName,
		hsize_t *dims,float32 *dsData_p,float32 *hourlyMin,float32 *hourlyMax);
		
int32 l4c_easegrid2_wgs84_fwd(const int32 mapProjectionCode,const float64 lat,const float64 lon, float64 *ithRow, float64 *ithColumn);
int32 l4c_easegrid2_wgs84_inv(const int32 mapProjectionCode,float64 ithRow, float64 ithColumn,float64 *lat,float64 *lon);
void  geog_forwardtransform(const float64 lat,const float64 lon, int32 *ithRow, int32 *ithCol);
void  geog_inverse(const int32 ithRow,const int32 ithCol,float64 *lat,float64 *lon);
int32 regrid_FP_to_EG_9km(const int32 nColsGeog,uint8 *pftImg_p,float32 *destEgImg_p,float32 *srcGeogImg_p);
int32 driverLogic_FP(SesCtl *sesCtl_p     ,char *dayOfYearStr);

hid_t soc_openfile(SesCtl *sesCtl_p,char *hdf5Path);
		
// L4_SM specific service functions
hid_t l4_sm_openfile(SesCtl *sesCtl_p,char *hdf5Path); 

int32 l4_sm_granuleHHMM_to_HrIndex(char *granulePath,char *hhmmString,char *yyyyStr);

void *l4_sm_get_dataset(hid_t file_id,char *datasetName,int *dRank,hsize_t *dims,int32 *nPix) ;
// L4_SM specific...
int32   l4_sm_verticalDrilldown(SesCtl *sesCtl_p,char *dayOfYearStr,char *datasetName) ;

// for L4_SM, returns hourly mean across 2D spatial domains
float32 l4_sm_summarizeVariable(FILE *logf,char *datasetName,hsize_t *dims,float32 *dsData_p,float32 *hourlyMin,float32 *hourlyMax) ;

int32   driverLogic_L4_SM(SesCtl *sesCtl_p  ,char *dayOfYearStr);


// L3_SM_A specific service functions
hid_t l3_sm_a_openfile(SesCtl *sesCtl_p,char *hdf5Path);
void *l3_sm_a_get_dataset(SesCtl *sesCtl_p,char *datasetName,int *dRank,hsize_t *dims);
// deprecated buildMask...
int32 l3_sm_a_ProcessFreezeThaw(SesCtl *sesCtl_p,uint16 *surfflag_p,uint8  *freezeThawQC_p);
int32 driverLogic_L3_SM_A(SesCtl *sesCtl_p,char *dayOfYearStr);
int32 l3_sm_a_granuleHHMM_to_HrIndex(char *granulePath,char *hhmmString,char *yyyyStr);

int32 l3_sm_a_freshenFreezeThawState(SesCtl *sesCtl_p,uint8 *freezeThawQC_p, uint8*ftRollingLast_p, uint8 *ftRollingImg_p);

// FP specific...
int32 fp_verticalDrilldown(SesCtl *sesCtl_p,char *dayOfYearStr,char *datasetName);


#endif


