# ----------------------------------------------------------------------
# file         : l4c_MODIS_NTSG.mak
# purpose      : Makefile for l4c_MODIS executable
# author       : Joe Glassy, K. Arthur Endsley
# platform     : ntsgcompute22017
# dependencies : libuuid.a, HDF5 v1.8.x, HDF4 v4.x, HDFEOS v2.x libraries
# notes        : n/a

# NOTE: This HDF4 works fine as a stand-alone HDF4 without HDFEOS but it does NOT
#   seem compatible with the HDFEOS library I'm using below...
#   HDF4INC = /usr/local/other/hdf4/4.2.6/gcc-4.1.2/include/hdf
#   HDF4LIB = /usr/local/other/hdf4/4.2.6/gcc-4.1.2/lib

# NOTE: these are all static libraries
# HDF4INC = /usr/local/other/hdf/4.2.5/include
# HDF4LIB = /usr/local/other/hdf/4.2.5/lib
# HDF4INC   = /discover/nobackup/jglassy/L4_C/include/h4
# HDF4LIB   = /discover/nobackup/jglassy/L4_C/lib/h4
# HDF5LIB   = /discover/nobackup/jglassy/L4_C/lib/h5
# HDF5INC   = /discover/nobackup/jglassy/L4_C/include/h5
# HDFEOS_LIB= /usr/local/hdfeos_v2.18/lib
# HDFEOS_INC= /usr/local/hdfeos_v2.18/include
#
# SZINC=/discover/nobackup/projects/gmao/share/dao_ops/Baselibs/v3.2.0_buildSLES11SP1_IMPI4/Linux/include/szlib
# LIBZINC=/discover/nobackup/projects/gmao/share/dao_ops/Baselibs/v3.2.0_buildSLES11SP1_IMPI4/Linux/include/zlib
# HDFEOSINC=/discover/nobackup/projects/gmao/share/dao_ops/Baselibs/v3.2.0_buildSLES11SP1_IMPI4/Linux/include/hdfeos
# HDF5INC=/discover/nobackup/projects/gmao/share/dao_ops/Baselibs/v3.2.0_buildSLES11SP1_IMPI4/Linux/include/hdf5
# HDF4INC=/discover/nobackup/projects/gmao/share/dao_ops/Baselibs/v3.2.0_buildSLES11SP1_IMPI4/Linux/include/hdf
# ALL_LIB=/discover/nobackup/projects/gmao/share/dao_ops/Baselibs/v3.2.0_buildSLES11SP1_IMPI4/Linux/lib

CC = gcc

CFLAGS =  -g -Wall -Wextra -fPIC -D_GNU_SOURCE -DHAVE_UUID -DHAVE_HDF4 -DHAVE_HDF5

# <JMG> 2020-12-17T09:48 updated for 2020 use
# <KAE> 2025-02-25 Commented out so that user-defined env vars can be used
# HDFEOS_ROOT=/usr/local/hdf-eos/hdf-eos_v2.18
# HDFEOS_LIB=$(HDFEOS_ROOT)/lib
# HDFEOS_INC=$(HDFEOS_ROOT)/include

# <KAE> 2021-07-21 Added for supporting VIIRS EOS-HDF5 granules
HDFEOS5_ROOT=/usr/local/hdf-eos/hdfeos5
HDFEOS5_LIB=$(HDFEOS5_ROOT)/lib/linux64
HDFEOS5_INC=$(HDFEOS5_ROOT)/include

LDFLAGS = -lfrm -lhdfeos -lhe5_hdfeos -lGctp -lmfhdf -ldf -lhdf5_hl -lhdf5 -lz -lsz -ljpeg -luuid -lm -ltirpc

INCS = -I$(FRM_API_INC) -I$(HDFEOS_INC) -I$(HDFEOS5_INC) -I$(HDF4_INC) -I$(HDF5_INC) -I.

LIBS = -L$(HDF5_LIB) -L$(FRM_API_LIB) -L$(HDFEOS_LIB) -L$(HDFEOS5_LIB) -L$(HDF4_LIB)

TARGET = l4c_MODIS.ex

SRCS = 	l4c_MODIS_rip.c \
	l4c_MODIS_FparQaBin.c \
	l4c_MODIS_bplut.c \
	l4c_MODIS_ses.c \
	l4c_MODIS_PkgMgrInit.c \
	l4c_MODIS_fx.c \
	l4c_MODIS_io.c \
	l4c_MODIS_easegrid.c \
	l4c_MODIS_logic.c \
	l4c_MODIS_drv.c \
	l4c_MODIS_WriteMetadata.c \
	l4c_MODIS.c

# Create one object file per .c source file
OBJS = $(SRCS:.c=.o)

all: $(TARGET)

# Re-arranged order: the order in the CC line is very important to correct linking...
# NOTE: LIBS refers to the -L locations, not the libraries themselves
# Prior: $(CC) $(CFLAGS) -o $(TARGET) $(OBJS) $(INCS) $(LDFLAGS) $(LIBS)
$(TARGET): $(OBJS)
	$(CC) $(CFLAGS) -o $(TARGET) $(OBJS)  $(INCS) $(LIBS) $(LDFLAGS)

test:
	./l4c_MODIS.ex l4c_MODIS.rip

clean:
	rm -f *.o *~ core $(INCDIR)*~

install:
	mv $(TARGET) ..

# Update the .o files whenever a newer .c file is encountered
.c.o:
	$(CC) $(CFLAGS) $(INCS) -c $< -o $@
