// ---------------------------------------------------------------
// function: l4c_MODIS_drv.c
// purpose : Implement main control, calling logic driver for l4c_MODIS, process a single 8-day period
// author  : joe glassy
// revised : 2013-11-06T17:04:00-07:00
// history : 2013-11-01
// notes   : n/a


// ---------------------------------------------------------------
#include "l4c_MODIS.h"
#include "l4c_MODIS_prot.h"

// ---------------------------------------------------------------
// function: l4c_MODIS_Driver()
// purpose : Implement main control, calling logic driver for l4c_MODIS, process a single 8-day period
// author  : joe glassy
// revised : 2013-11-06T17:04:00-07:00
// history : 2013-11-01
// notes   : n/a
int l4c_MODIS_Driver(void)
{
    SesCtl *sesCtl_p   = l4c_GetSesCtl() ;
    NsbObj *nsbObj_p   = l4c_GetNsbOverseer(); 
    ImgPkg *imagePkg_p = l4c_GetImagePackage();
  
    // set up equiv symbols, but as pointers, that look more like the legacy ones
    ease_struct    *ease_p = NULL ; // set convenience pointers
    
    int32 ithRow = 0 ;
    int32 ithCol = 0 ;
    int32 serialIdx = 0;
 
    int   nTiles = 0 ;
    char yeardoyString[32] = {0} ;
    char datestr[32]       = {0} ;
    // char **fparTileList = NULL;
    char **inputTileList= NULL; // <JMG> may point to FPAR or NDVI list
    char fxName[256] = {0} ;
    char hdf5Path[FILENAME_MAX] = {0};
    char nvpBuf[FILENAME_MAX] ;
 
    // name to register this function
    strcpy(fxName,(char *)__func__); 

    // l4c_MODIS.exe Processes only a single 8-day compositing period
    xLogMsg("@A In(%s) l4c_MODIS_Driver executing~\n",fxName);

    // Read through file lists of input 8-days (one list of files per 8-day period)
    //  datestr looks something like 2013.05.25, where '2013.05.25' is a file naming an 8-day period, that
    //  itself contains a list of 286 tiles for that 8-day period
    
    // Retrieve list of all nFparGranules (e.g.286 odd) FPAR 8-DAY tiles parsed from RIP file
    inputTileList = sesCtl_p->input_Fpar_pathList ;
    nTiles = sesCtl_p->nFparGranules ;
    
    // Allocate memory for OUTPUTS -- EASEGRID FPAR, FPAR QC and MASK fpar, qc, mask
    l4c_AllocateFparEasegrid();
        
    // set pointer into populated EG 1km container
    ease_p    = &imagePkg_p->ease ; 

    // <JMG> 2021-07-14, DEPRECATE branch on NDVI, use same for MODIS or VIIRs now
    // Initialize matrices to arbitrary defaults
    xLogMsg("@A In(%s) : initializing EASEGRID (fpar,qc) to fill value %d~ \n",fxName, FPAR_FILL_VALUE);
    
    for (ithRow=0; ithRow < EGv2_GLOBAL_1KM_NS_PIXELS; ithRow++)
    {
        for (ithCol =0; ithCol < EGv2_GLOBAL_1KM_EW_PIXELS; ithCol++)
        {
            serialIdx = M_2D_B0(ithRow, ithCol, EGv2_GLOBAL_1KM_EW_PIXELS);
            // <JMG> 2021-07-14 DEPRECATED nvdi handling, no branch needed now
            ease_p->fpar[serialIdx]  = FPAR_FILL_VALUE;
            ease_p->qc[serialIdx]    = QA_FILL_VALUE ; // still 255 same as FPAR for now

        } // end:: iCol loop
    } // end:: iRow loop

    // Regrid data from 1km sinusoidal to 1km ease v2
    //   Call interface documentation:
    //   -- datestr -- contains something like 2013/05.25
    //   -- cont_p  -- who knows?
    //   -- replaced fnames[] with **fparTileList string-table
    //   -- nTiles , counts of MODIS files to process
    //   -- ease_p , pointer to EASEGRID 1km container, for output images fpar,qc,mask
    xLogMsg("@A In(%s) calling tile_loop next ~\n",fxName);
    
    // LOOP ITERATE: process nTiles of input
    tile_loop(datestr, yeardoyString, inputTileList, nTiles);

    xLogMsg("@A In(%s) tile_loop COMPLETED ~\n",fxName);

    // Set ocean/ice sheets to fill value, based on lc mask
    xLogMsg("@A In(%s) calling set_land_mask_1km to mask out all non-land pixels on EASEGRID using EG 1km mask~ \n",fxName);
    // set_land_mask_1km(ease_p->mask, ease_p->fpar, ease_p->qc, ease_p->qc_extra);
    set_land_mask_1km(ease_p->mask, ease_p->fpar, ease_p->ndvi, ease_p->qc, ease_p->ndvi_qc);
    
    // WRITE all outputs now
    xLogMsg("@A In(%s) l4c_writeSmapOutputs next ~\n",fxName);
    l4c_writeSmapOutputs();

 return 0 ;
 
} // end::l4c_MODIS_Driver()

