/* --------------------------------------------------------------------
 l4c_daily_gridconst.h
 Revised: 2021-07-14T11:08, EvalDiag in prep for VIIRs integration
 Revised: 2013-05-19T10:44:00-07:00
 Author : joe glassy, Lupine Logic Inc.
 Version: 4.0
 Purpose: define all L4_C grid constants
-------------------------------------------------------------------- */

#ifndef L4C_GridConstDef 
#define L4C_GridConstDef 1

// GRID CONSTANTS
// Define EASE-Grid 2.0 spatial extents constants 
#define EG_1KM_ROWS       14616
#define EG_1KM_COLS       34704
#define EG_1KM_CELLS  507233664

#define EG_9KM_ROWS        1624
#define EG_9KM_COLS        3856
#define EG_9KM_CELLS    6262144

// EASE-Grid v2 1km CELL constants 
#define EGv2_GLOBAL_1KM_EW_PIXELS      34704   
#define EGv2_GLOBAL_1KM_NS_PIXELS      14616
#define EGv2_GLOBAL_1KM_PIXELS     507233664 
// EASE-Grid v2 9km CELL constants
#define EGv2_GLOBAL_9KM_EW_PIXELS       3856
#define EGv2_GLOBAL_9KM_NS_PIXELS       1624
#define EGv2_GLOBAL_9KM_PIXELS       6262144

// MODIS sinusoidal CELL or PIXEL related constants 
// Define MODIS (collection 5 and later) sinusoidal CELL or PIXEL constants
#define MODIS_GLOBAL_1KM_EW_PIXELS        43200
#define MODIS_GLOBAL_1KM_NS_PIXELS        21600
#define MODIS_GLOBAL_1KM_SUM_PIXELS   933120000
// <JMG> 2017-01-26T12:55, Define MODIS (Collection 6) sinusoidal CELL and/or PIXEL constants
#define MODIS_GLOBAL_500M_EW_PIXELS          86400
#define MODIS_GLOBAL_500M_NS_PIXELS         43200
#define MODIS_GLOBAL_500M_SUM_PIXELS   3732480000

// <LAJ> 2017-02-01 added: 
#define MODIS_1KM_NESTED_500M_EW_PIXELS  2
#define MODIS_1KM_NESTED_500M_NS_PIXELS  2  
#define MODIS_1KM_NESTED_500M_PIXELS     4 

// define MODIS Collection 5 10-degree TILE pixel parameters
#define MODIS_1KM_EW_PIXELS      1200 
#define MODIS_1KM_NS_PIXELS      1200 
#define MODIS_1KM_SUM_PIXELS     1440000
// define MODIS Collection 6 500m TILE pixel parameters
#define MODIS_500M_EW_PIXELS      2400
#define MODIS_500M_NS_PIXELS      2400
#define MODIS_500M_SUM_PIXELS     5760000

// Misc other grid and coordinate transformation constants of interest. Not used yet.
#define MODIS_VV_TILES                      18
#define MODIS_HH_TILES                      36
#define MODIS_1KM_GLOBAL_ROWS            21600
#define MODIS_1KM_GLOBAL_COLS            43200
#define MODIS_1KM_GLOBAL_PIXELS      933120000

// MERRA 
#define MERRA_ROWS        361
#define MERRA_COLS        540
// GEOS5 Forward Processing is geographic 1/4 degree resolution
// 1-deg is {-90..+90}=180 along lat axes, {-180...+180}=360 along longitudinal axes
#define FP_GEOS5_ROWS     720
#define FP_GEOS5_COLS    1400

#endif
