// ---------------------------------------------------------------
// function: l4c_MODIS_metadata.c
// purpose : Implement metadata for l4c_MODIS
// author  : joe glassy
// revised : 2013-11-06T17:04:00-07:00
// history : 2013-11-01
// notes   : n/a
// ---------------------------------------------------------------

#include "l4c_MODIS.h"
#include "l4c_MODIS_prot.h"

char *l4cMetadataCatalog[] = {
"Element-name,Shape,Data-type,Description",
"InputName,scalar,string,The name of one or more data granules in the SMAP SDS that were used to generate this product.  Products stored at and/or distributed by an ECS facility may include an InputPointer with a complete path instead. ",
"AncillaryInputName,AncFile_Array,string,An array of file names that specifies all of the ancillary data files that were used to generate this output product. Ancillary data include all SPS input other than those listed in the InputName.",
"SizeMBECSDataGranule,scalar,real,The size of this data granule in Megabytes.",
"ReprocessingPlanned,scalar,string,Specifies whether additional reprocessing is planned.  Contains adequate bytes to specify a short rationale, if its needed.",
"GranuleName,scalar,string,The name of the product file. ",
"ProductionDateTime,scalar,string,The date and time when the product was created.",
"SPSVersion,scalar,string,Indicator of the version of the operational SPS.  This version specifier covers both the SPS software as well as the configuration and environment specifiers.",
"QAGranuleName,scalar,string,A pointer to the quality assurance product that was generated with this product.",
"ParameterName,Parameter_Array,string,Product parameter used as a basis for Quality Assurance measure.  The ECS model permits the use of more than one of parameter for QA measure per granule",
"QAPercentMissingData,Parameter_Array,real,The percent of missing data elements in array(s) that represent the parameter specified in ParameterName.  SMAP will list the percent as a float instead of an integer as specified by the Earth Science Data Model.",
"QAPercentOutOfBoundsData,Parameter_Array,real,The percent of elements in the array(s) specified in ParameterName that are out of expected bounds.  SMAP will list the percent as a float instead of an integer as specified by the Earth Science Data Model.",
"RangeBeginningDate,scalar,string,The date when the earliest data contained in the product were acquired.",
"RangeEndingDate,scalar,string,The date when the latest data contained in the product were acquired.",
"RangeBeginningTime,scalar,string,The time when the earliest data contained in the product were acquired.",
"RangeEndingTime,scalar,string,The time when the latest data contained in the product were acquired.",
"LongName,scalar,string,A complete descriptive name for the product.",
"ShortName,scalar,string,'L4_C' - The short name used to identify all of the individual products of a given product type.",
"CollectionDescription,scalar,string,Text or label that associates all of the data files in a single collection",
"CompositeReleaseID,scalar,string,Identifier that specifies the sum total of all variables that might impact product output.",
"SWVersionID,scalar,string,Identifies the version of the SPS software that was used to generate the associated product. SPS software includes all software modules that operate with the SPS.  This identifier does not consider configuration or environment changes.",
"AlgorithmVersionID,scalar,string,Identifies the version of the SPS algorithm that was used to generate the associated product.  This identifier does not consider changes to input, non-algorithmic software, or environment changes.",
"ParameterVersionID,scalar,string,Identifies the set of parameters that were used to generate the associated product.  This identifier does not consider changes to code or run input.",
"SPSName,scalar,string,Indicates the formal SPS name.",
"SPSIdentifier,scalar,string,Indicates SPS pipeline level.  For example, one level might be Radar Level 1B.",
"SPSDateLastModified,scalar,string,The date of implementation of the new SPSVersion specified in the ECSDataGranule class.  The date when SMAP began processing the SPS software in the environment used to generate the associated product.",
"SWDateLastModified,scalar,string,The date of implementation of the SWVersion specified in the AlgorithmPackage class. The date when SMAP began processing the SPS software that was used to generate the associated product.",
"PlatformShortName,scalar,string,A short name for the platform associated with the instrument which acquires the accompanying data.",
"PlatformType,scalar,string,'spacecraft' - The type of platform associated with the instrument which acquires the accompanying data",
"PlatformLongName,scalar,string,A longer, more detailed name for the platform associated with the instrument which acquires the accompanying data",
"InstrumentShortName,scalar,string,'SMAP' - A short name of the instrument that acquired the telemetry data.",
"InstrumentLongName,scalar,string,Soil Moisture Active/Passive - A longer, more detailed name for the instrument that acquired the telemetry data.",
"PSDName,scalar,string,The name of the document describing the contents of the product.",
"PSDVersion,scalar,string,The version of the document describing the contents of the product.",
"GapStartDateTime,Gap_Array,string,A specific date and time which indicates where a nonexistent, unnecessary, spurious, questionable, or erroneous data segment begins. ",
"GapStopDateTime,Gap_Array,string,A specific date and time which indicates where a nonexistent, unnecessary, spurious, questionable, or erroneous data segment ends. ",
"NumberOfGaps,scalar,integer,The number of gaps detected in the data product.  Equal to the number of elements in GapStartDateTime and GapStopDateTime arrays.",
"ProducerAgency,scalar,string,'NASA' - Identification of the agency that provides the project funding",
"ProducerInstitution,scalar,string,'JPL' - Identification of the institution that provides project management.",
"ProductionLocation,scalar,string,Facility in which the data product was produced",
"ProductionLocationCode,scalar,character,One-letter code indicating the ProductionLocation.",
"ProjectId,scalar,string,SMAP - The project identification string",
"DataFormatType,scalar,string,'NCSA HDF' or 'HDF EOS' - A character string that describes the internal format of the data product.",
"HDFVersionId,scalar,string,'5.x' - A character string that identifies the version of the HDF (Hierarchical Data Format) software that was used to generate this data file.",
"MapProjectionName,scalar,string,The name used to identify the systematic representation of all or part of the Earth's surface on a plane or developable surface.",
"GridCoordinateSystemName,scalar,string,Identification of the plane-rectangular coordinate system that was used to subdivide data in this product.",
"PermanentWaterBodyThreshold,scalar,real,Threshold value that indicates a significant fraction of the grid cell is over a permanent water body.",
"TransientWaterBodyThreshold,scalar,real,Threshold value that indicates a significant fraction of the grid cell is over a transient water body.",
"UrbanAreaThreshold,scalar,real,Threshold value that indicates a significant fraction of the grid cell is over one or more urban regions.",
"MountainousTerrainThreshold,scalar,real,Threshold value that indicates high variability of land elevation within the grid cell.",
"DenseVegetationThreshold,scalar,real,Threshold value that indicates that a significant fraction of the grid cell is covered with dense vegetation.",
"OpenWaterThreshold,scalar,real,Threshold value that indicates a significant fraction of the grid cell is over an open water body.",
"VegetationTypeTable,Veg type parameters array,real,Table of various ecosystem response characteristics that are used for NEE retrieval based on the predominant vegetation type.",
"SoilDecompositionFactor,scalar,real,Factor used to calculate soil decomposition rate for all biomes.  Reference Kmx in the Level 4 Carbon Model Assimilation ATBD.",
"GridSpacing,scalar,real,Specifies the size of the grid cells that are listed in this product. - 9 km.",
"AlgorithmPackageVersionID,scalar,string,Identifies the version of the implemented algorithm.",
"AlgorithmPackageAcceptanceDate,scalar,string,Specifies the date that the associated algorithm package version successfully passed Integration and Test procedures.",
"AlgorithmPackageMaturityCode,scalar,string,Values that indicate the relative maturity of the algorithm in the SPS.  Suggested values include Pre-launch, Prelim for preliminary, OPL for operational, stable and final.",
"AlgorithmDescriptor,scalar,string,A set of character strings which list important details about the implementation of the algorithm that was used to generate this product",
"DigitalElevationModel,scalar,string,Identifier of the Digital Elevation Map or model that was used to provide geolocation information in this product.",
"EllipsoidName,scalar,string,'WGS84' - Identification for the Geoid Model used to provide geolocation information in this product.",
"TimeVariableEpoch,scalar,string,Specification of the epoch that the product employs to represent all data elements that appear in the product as counts in seconds.",
"EpochJulianDate,scalar,real,Julian date associated with the epoch that the product employs to specify time elements that are represented as counts in seconds.",
"EpochUTCDateTime,scalar,string,UTC date and time associated with epoch the product employs to specify time elements that are represented as counts in seconds.",
"PlantFunctionTypes,Plant Function Type Array,string,Specification of the plant function types that appear in the Level 4 C Carbon Product"
} ;


// ---------------------------------------------------------------
// function: l4c_CreateSmapMetadata()
// purpose : writes SMAP attributes to HDF5 file
// author  : joe glassy
// revised : 2013-11-06T17:04:00-07:00
// notes   : This is placeholder code to serve as a reminder of the set of metadata elements
//           eventually going in to the output HDF. note that these will need to be posted in the
//           official nested group hierarchy when in operation.
//
hid_t l4c_CreateSmapMetadata(hid_t file_id)
{
 char lineBuffer[2048]      = {0} ;
 char keywordToken[512]     = {0} ;
 char valueToken[2048]      = {0} ;
 char *ch_p = NULL   ; // qualified token pointer
 char *brkb_p = NULL ; // pointer required for strtok_r thread safety
 char tokenSepChars[8] = "," ; // allowable token delimiters: blank,tab,space,'=' symbol
 int  ithColumn = 0;
 hid_t attr_id = -1;
 
 int  nMetadataElements = 65 ;
 int  m =0;

	// CREATE PLACEHOLDER SET OF METADATA Elements
	xLogMsg("@A l4c_CreateSmapMetadata");
 
 
	for(m=0; m < nMetadataElements ; m++)
	{
		ithColumn = 0 ;
		// spin through the delimited tokens on this RIP file line
		for(ch_p = strtok_r(lineBuffer,tokenSepChars,&brkb_p);
			ch_p != NULL;
			ch_p = strtok_r(NULL,tokenSepChars,&brkb_p))
		{
			if(ithColumn == 0)
			{
				memset(keywordToken,'\0',512);
				strcpy(keywordToken,ch_p);
				// must zero out valueToken each time here!
				memset(valueToken,'\0',2048);
			} else if(ithColumn >= 1) {
				if(ithColumn==1)
				{
					strcpy(valueToken,ch_p);
				} else {
					strcat(valueToken, "|");
					strcat(valueToken, ch_p);
				}
			}
		} // end:: ch_p token iterator on a single line
		
		// create the (placeholder) attribute now
		attr_id = l4c_createHdf5Attribute(file_id,keywordToken,valueToken);
		
	} // end:: m loop through metadata elements

 return attr_id ;
 
} // end::l4c_CreateSmapMetadata()
